#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gui.h>
#include <stdlib.h>
#include <string.h>

#include <plugin.h>
#include <stdio.h>
#include <node.h>
#include <floor.h>
#include <pluginmanager.h>
#include <guisupport.h>

GSList *floorList = NULL;

static GtkWidget *nodeCList;
static GtkWidget *nodeWindow;
static int selected_row = -1;

extern int node_width;
extern int node_height;
extern Floor *currentFloor;

void init_nodes_before_connect(Floor *floor){
    GList *nodeNode;
    Node *node;
    int i;
    nodeNode = g_list_first(floor->nodeList);
    while(nodeNode){
	node = (Node *)nodeNode->data;
	node->processed = FALSE;
	node->isReadyForProcessing = FALSE;
	for(i=0;i<node->numInputs;i++)
	    node->readyBuffers[i] = FALSE;
	nodeNode = nodeNode->next;
    }
}



gint floor_destroy(Floor *floor){
    //
    ////////////IMPLEMENT
    //
    //
    return TRUE;
}
void floor_remove_node(Floor *floor,Node *node){
    if(!IS_NODE_TYPE_MAIN_INPUT(node->nodeType) && 
       !IS_NODE_TYPE_MAIN_OUTPUT(node->nodeType)){
	node_disconnect_outputs(node);
	node_disconnect_inputs(node);
	floor->nodeList = g_list_remove(floor->nodeList,node);
	node_destroy(node);
	floor_repaint(floor);
    }
}

Floor *get_floor_from_drawingArea(GtkWidget *drawingArea){
   /*GSList *floorNode = floorList;
   Floor *floor;
   while(floorNode){
      floor = (Floor *)floorNode->data;
      if(floor->drawingArea == drawingArea) return floor;
      floorNode = floorNode->next;
   }
   fprintf(stderr, "PANIC!! I am about to return a NULL pointer... probably bad\n");
   return NULL;*/
   return currentFloor;
}

gint floor_repaint(Floor *floor){
    GList *nodeNode;
    Node *node;
    GdkRectangle updateRect;
    gdk_draw_rectangle(floor->pixmap, floor->bgColor,
	    		TRUE, 0, 0,
			floor->drawingArea->allocation.width,
			floor->drawingArea->allocation.height);

    nodeNode = g_list_first(floor->nodeList);
    while(nodeNode){
	node = (Node *)nodeNode->data;
	draw_node_in_floor(node,floor);
	nodeNode = nodeNode->next;
    }
    if(floor->connectingToInput){
    gdk_draw_line(floor->pixmap, floor->blackColor,
	    ((Node *)floor->outputConnection)->position.x+node_width,
	    ((Node *)floor->outputConnection)->position.y+(int)(.5 * node_height),
	    floor->cursor.x,
	    floor->cursor.y);
    }
    else if(floor->connectingToOutput){
	gdk_draw_line(floor->pixmap, floor->blackColor,
	    ((Node *)floor->inputConnection)->position.x,
	    ((Node *)floor->inputConnection)->position.y+(int)(.5 * node_height),
	    floor->cursor.x,
	    floor->cursor.y);

    }

    updateRect.x = 0;
    updateRect.y = 0;
    updateRect.width = floor->drawingArea->allocation.width;
    updateRect.height = floor->drawingArea->allocation.height;
    gtk_widget_draw(floor->drawingArea,&updateRect);
    return 0;	    
}

gint add_node_to_floor_with_name(Floor *floor, gchar *pluginName){
    Node *node;
    /* create a new node */
    node = node_new_with_name(pluginName);
    /* add it to the list */
    floor->nodeList = g_list_append(floor->nodeList,node);
    floor_repaint(floor);
    return TRUE;
}


gint add_node_to_floor(Floor *floor){
    Node *node;
    /* create a new node */
    node = node_new();
    /* add it to the list */
    floor->nodeList = g_list_append(floor->nodeList,node);
    floor_repaint(floor);
    return TRUE;
}

static gint floor_expose_event(GtkWidget *widget, GdkEventExpose *event){
    Floor *floor;
    floor = get_floor_from_drawingArea(widget);

    gdk_draw_pixmap(floor->drawingArea->window,
                  floor->drawingArea->style->fg_gc[GTK_WIDGET_STATE (widget)],
                  floor->pixmap,
                  event->area.x, event->area.y,
                  event->area.x, event->area.y,
                  event->area.width, event->area.height);

    return FALSE;
}

GdkColor *NewColor(long red, long green, long blue){
    GdkColor *c = (GdkColor *)malloc(sizeof(GdkColor));
    c->red = red;
    c->green = green;
    c->blue = blue;
    gdk_color_alloc(gdk_colormap_get_system(),c);
    return(c);
}

GdkGC *GetPen (Floor *floor,GdkColor *c){
    GdkGC *gc;
    gc = gdk_gc_new(floor->pixmap);
    gdk_gc_set_foreground(gc,c);
    return(gc);
}



static gint floor_configure_event(GtkWidget *widget, GdkEventConfigure *event){
    Floor *floor;
    floor = get_floor_from_drawingArea(widget);
    if(floor->pixmap)
	gdk_pixmap_unref(floor->pixmap);

    floor->pixmap =  gdk_pixmap_new(widget->window,
	    widget->allocation.width,
	    widget->allocation.height,
	    -1);
    gdk_draw_rectangle(floor->pixmap,
	    widget->style->white_gc,
	    TRUE,
	    0, 0,
	    widget->allocation.width,
	    widget->allocation.height);

    if(floor->bgColor == NULL){
	floor->bgColor = GetPen(floor,NewColor(0xffff00,0xffff00,0xffeefe));
	floor->linksColor = GetPen(floor,NewColor(0xffffff,0xffffaa,0xffeefe));
	floor->nodesColor =  GetPen(floor,NewColor(50372,50372,37265));
	floor->fontColor =  GetPen(floor,NewColor(0xffff,0xffff,0xffeefe));
	floor->blackColor = GetPen(floor,NewColor(0x000000,0x000000,0x000000));
	floor->whiteColor = GetPen(floor,NewColor(0xffffff,0xffffff,0xffffff));

    }
    floor_repaint(floor);
    return TRUE;
}

static gint floor_button_release_event(GtkWidget *widget, GdkEventButton *event){
    Floor *floor;
    GList *nodeNode;
    GdkPoint point;
    NodeSection section;

    floor = get_floor_from_drawingArea(widget);
    
    if (event->button == 1 && floor->pixmap != NULL){
	/* we are not dragging anymore */
	floor->dragging = FALSE;
	if(floor->connectingToInput){
	    /* if we were trying to connect to the input of a node */
	    /* first, check if we are in the node part of a node */
	    point.x = (int)event->x;
	    point.y = (int)event->y;
	    nodeNode = g_list_last(floor->nodeList);
	    while(nodeNode && point_touches_node(point,nodeNode->data)!=TRUE)
		nodeNode = nodeNode->prev;
	    if(nodeNode!=NULL){
		/* released the mouse in the node nodeNode->data. */
		section = point_touches_node_section(point,nodeNode->data);
		if(section == NODE_SECTION_INPUT){
		    /* ok, released the mouse in the input section of a
		     * node
		     */
		    /* is there a slot available for connecting ?? */
		    if(node_input_available(nodeNode->data)==TRUE){
			/* Yes !! we can connect them */
			node_connect_input_to_output(nodeNode->data,floor->outputConnection);
			floor->outputConnection = NULL;
			floor->connectingToInput = FALSE;
			/* finally repaint so we can see what we did */
			floor_repaint(floor);
		    }
		}
	    }
	}else if(floor->connectingToOutput){
	    point.x = (int)event->x;
	    point.y = (int)event->y;
	    nodeNode = g_list_last(floor->nodeList);
	    while(nodeNode && point_touches_node(point,nodeNode->data)!=TRUE)
		nodeNode = nodeNode->prev;
	    if(nodeNode!=NULL){
		section = point_touches_node_section(point,nodeNode->data);
		if(section == NODE_SECTION_OUTPUT){
		    if(node_output_available(nodeNode->data)==TRUE){
			//fprintf(stderr,"Trying to perform connection\n");
			node_connect_input_to_output(floor->inputConnection,nodeNode->data);
			floor->inputConnection = NULL;
			floor->connectingToOutput = FALSE;
			floor_repaint(floor);
		    }
		}

	    }
	}
    }else if(event->button == 3 && floor->pixmap != NULL){
	//fprintf(stderr,"Button 2 released on %d, %d\n",(int)event->x,(int)event->y);
    }
    floor->connectingToOutput = FALSE;
    floor->connectingToInput = FALSE;
    floor_repaint(floor);
    return TRUE;
}
static gint floor_button_press_event(GtkWidget *widget, GdkEventButton *event){
    Floor *floor;
    GdkPoint point;
    GList *nodeNode;
    NodeSection section;

    floor = get_floor_from_drawingArea(widget);

    if (event->button == 1 && floor->pixmap != NULL){
	/* There is a click with the left button */
	//fprintf(stderr,"Button 1 clicked on %d, %d\n",(int)event->x,(int)event->y);
	point.x = (int)event->x;
	point.y = (int)event->y;
	nodeNode = g_list_last(floor->nodeList);
	while(nodeNode && point_touches_node(point,nodeNode->data)!=TRUE)
	    nodeNode = nodeNode->prev;
	if(nodeNode!=NULL){
	    /* it touched the node nodeNode->data. */
	    section = point_touches_node_section(point,nodeNode->data);
	    switch(section){
		case NODE_SECTION_BODY:	
		    floor->draggingObject = nodeNode->data;
		    floor->dragging = TRUE;
		    floor->objectDifference.x = ((Node *)nodeNode->data)->position.x - (int)event->x;
		    floor->objectDifference.y = ((Node *)nodeNode->data)->position.y - (int)event->y;
		    break;
		case NODE_SECTION_INPUT:
		    if(node_input_available(nodeNode->data)==TRUE){
			floor->connectingToOutput = TRUE;
			floor->inputConnection = nodeNode->data;
			//fprintf(stderr,"Trying to start the connection\n");
		    }
		    break;
		case NODE_SECTION_OUTPUT:
		    /* first check if we can add a connection here */
		    if(node_output_available(nodeNode->data)==TRUE){
			/* yes it is available */
			floor->connectingToInput = TRUE;
			floor->outputConnection = nodeNode->data;
		    }
		    break;
		case NODE_SECTION_NONE:
		default:
		    //fprintf(stderr,"touched the node, but point_touches_node_section returned none ????\n");
	    }
	}

    }else if(event->button == 3 && floor->pixmap != NULL){
	//fprintf(stderr,"Button 3 clicked on %d, %d\n",(int)event->x,(int)event->y);
	point.x = (int)event->x;
	point.y = (int)event->y;
	nodeNode = g_list_last(floor->nodeList);
	while(nodeNode && point_touches_node(point,nodeNode->data)!=TRUE)
	    nodeNode = nodeNode->prev;
	if(nodeNode!=NULL){
	    /* it touched the node nodeNode->data. */
	    section = point_touches_node_section(point,nodeNode->data);
	    switch(section){
		case NODE_SECTION_BODY:
		    floor->activeNode = nodeNode->data;
		    gtk_menu_popup(GTK_MENU(floor->popUpMenuOnNode),NULL,NULL,NULL,NULL,
			    event->button, event->time);
		    break;
		default:
		    break;
	    }
	}else{
	    /* show the general menu */
	    gtk_menu_popup(GTK_MENU(floor->popUpMenuGeneral),NULL,NULL,NULL,NULL,
		    event->button, event->time);
	}
    }
    return TRUE;
}

static gint floor_motion_notify_event(GtkWidget *widget,GdkEventMotion *event){
    Floor *floor;
    int x, y;
    GdkModifierType state;

    floor = get_floor_from_drawingArea(widget);

    if (event->is_hint)
	gdk_window_get_pointer (event->window, &x, &y, &state);
    else
    {
	x = event->x;
	y = event->y;
	state = event->state;
    }

    if (state & GDK_BUTTON1_MASK && floor->pixmap != NULL){
	//fprintf(stderr,"Button 1 draging on %d, %d\n",x,y);
	if(floor->dragging == TRUE){
	    /* for now we only drag nodes */
	    ((Node *)floor->draggingObject)->position.x = x + floor->objectDifference.x;
	    ((Node *)floor->draggingObject)->position.y = y + floor->objectDifference.y;
	    floor_repaint(floor);
	}else if(floor->connectingToInput){
	    floor->cursor.x = x;
	    floor->cursor.y = y;
	    floor_repaint(floor);
	}else if(floor->connectingToOutput){
	    floor->cursor.x = x;
	    floor->cursor.y = y;
	    floor_repaint(floor);
	}

    }else if(state & GDK_BUTTON3_MASK && floor->pixmap != NULL){
	/*fprintf(stderr,"Button 2 draging on %d, %d\n",x,y);*/
    }else if (floor->pixmap != NULL){

    }
    return TRUE;
}

static gint add_node_cancel_clicked(GtkWidget *widget, gpointer data){
    gtk_widget_destroy(GTK_WIDGET(nodeWindow));
    nodeCList = NULL;
    selected_row = -1;
    return TRUE;
}

static gint add_node_ok_clicked(GtkWidget *widget, gpointer data){
    gchar *pluginName;
    if(selected_row!=-1){
	gtk_clist_get_text(GTK_CLIST(nodeCList),selected_row,0,&pluginName);
	add_node_to_floor_with_name((Floor *)data,pluginName);
    }
    gtk_widget_destroy(GTK_WIDGET(nodeWindow));
    nodeCList = NULL;
    selected_row = -1;
    return TRUE;
}

static gint add_node_select_row(GtkWidget *list, gint row, gint column, GdkEventButton *event, gpointer data){
    selected_row = row;
    return TRUE;
}
int add_node_menu_activate(GtkWidget *widget, gpointer data){
    /* show a dialog with the available pluging*/
    Floor *floor = (Floor *)data;
    GtkWidget *scrolledWindow;
    GtkWidget *vbox,*hbox;
    GtkWidget *okButton,*cancelButton;
    gchar *titles[] = {"Stompbox Name","Description"};
    
    nodeWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_modal(GTK_WINDOW(nodeWindow),TRUE);
    gtk_window_set_title(GTK_WINDOW(nodeWindow),"Stompbox Selection");
    
    vbox = gtk_vbox_new(FALSE,10);
    gtk_container_add(GTK_CONTAINER(nodeWindow),vbox);

    nodeCList = gtk_clist_new_with_titles(2,titles);
    gtk_clist_set_selection_mode(GTK_CLIST(nodeCList), GTK_SELECTION_SINGLE);
    gtk_signal_connect(GTK_OBJECT(nodeCList), "select_row", 
	    GTK_SIGNAL_FUNC(add_node_select_row), NULL);
    find_all_plugins(nodeCList);
    
    scrolledWindow = gtk_scrolled_window_new(NULL,NULL);
    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolledWindow),nodeCList);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledWindow),GTK_POLICY_AUTOMATIC,
	    			   GTK_POLICY_AUTOMATIC);
    gtk_widget_set_usize(GTK_WIDGET(scrolledWindow),300,200);
    gtk_box_pack_start(GTK_BOX(vbox),scrolledWindow,FALSE,FALSE,0);
    
    hbox = gtk_hbox_new(TRUE,10);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE,FALSE,0);

    okButton = gtk_button_new_with_label("OK");
    gtk_signal_connect(GTK_OBJECT(okButton),"clicked",
	    	GTK_SIGNAL_FUNC(add_node_ok_clicked),floor);
    gtk_box_pack_start(GTK_BOX(hbox),okButton,TRUE,TRUE,0);

    cancelButton = gtk_button_new_with_label("Cancel");
    gtk_signal_connect(GTK_OBJECT(cancelButton),"clicked",
	    GTK_SIGNAL_FUNC(add_node_cancel_clicked),floor);
    gtk_box_pack_start(GTK_BOX(hbox), cancelButton, TRUE, TRUE, 0);
    
    gtk_widget_show_all(GTK_WIDGET(nodeWindow));
/*    add_node_to_floor((Floor *)data);
 */
    return TRUE;
}

static int delete_node_menu_activate(GtkWidget *widget, gpointer data){
    Floor *floor = (Floor *)data; 
    if(floor->activeNode!=NULL)
	floor_remove_node(floor,floor->activeNode);
    floor->activeNode = NULL;
    return TRUE;
}

static int disconnect_outputs_node_menu_activate(GtkWidget *widget, gpointer data){
    Floor *floor = (Floor *)data;
    if(floor->activeNode!=NULL)
	node_disconnect_outputs(floor->activeNode);
    floor->activeNode = NULL;
    floor_repaint(floor);
    return TRUE;
}

static int disconnect_inputs_node_menu_activate(GtkWidget *widget, gpointer data){
    Floor *floor = (Floor *)data;
    if(floor->activeNode!=NULL)
	node_disconnect_inputs(floor->activeNode);
    floor->activeNode = NULL;
    floor_repaint(floor);
    return TRUE;
}

static int configure_node_menu_activate(GtkWidget *widget, gpointer data){
    Floor *floor = (Floor *)data;
    if(floor->activeNode!=NULL)
	node_configure_plugin(floor->activeNode);
    floor->activeNode = NULL;
    return TRUE;
}


Floor *gui_floor_new(){
    Floor *floor;
    Node *defaultNodes;
    GtkWidget *popUpMenuItems;
    floor = (Floor *)calloc(sizeof(Floor),1);
    /* set up the general popup menu */
    floor->popUpMenuGeneral = gtk_menu_new();
    popUpMenuItems = gtk_menu_item_new_with_label("Add A StompBox");
    gtk_signal_connect(GTK_OBJECT(popUpMenuItems),"activate",
	    GTK_SIGNAL_FUNC(add_node_menu_activate),floor);
    gtk_menu_append(GTK_MENU(floor->popUpMenuGeneral),popUpMenuItems);
    gtk_widget_show(popUpMenuItems);

    /* set up the onNode popup menu */
    floor->popUpMenuOnNode = gtk_menu_new();
    popUpMenuItems = gtk_menu_item_new_with_label("Adjust Stompbox Settings");
    gtk_signal_connect(GTK_OBJECT(popUpMenuItems),"activate",
	    GTK_SIGNAL_FUNC(configure_node_menu_activate),floor);
    gtk_menu_append(GTK_MENU(floor->popUpMenuOnNode),popUpMenuItems);
    gtk_widget_show(popUpMenuItems);
    popUpMenuItems = gtk_menu_item_new_with_label("Delete Stompbox");
    gtk_signal_connect(GTK_OBJECT(popUpMenuItems),"activate",
	    GTK_SIGNAL_FUNC(delete_node_menu_activate),floor);
    gtk_menu_append(GTK_MENU(floor->popUpMenuOnNode),popUpMenuItems);
    gtk_widget_show(popUpMenuItems);
    popUpMenuItems = gtk_menu_item_new_with_label("Disconnect Inputs");
    gtk_signal_connect(GTK_OBJECT(popUpMenuItems),"activate",
	    GTK_SIGNAL_FUNC(disconnect_inputs_node_menu_activate),floor);
    gtk_menu_append(GTK_MENU(floor->popUpMenuOnNode),popUpMenuItems);
    gtk_widget_show(popUpMenuItems);
    popUpMenuItems = gtk_menu_item_new_with_label("Disconnect Outputs");
    gtk_signal_connect(GTK_OBJECT(popUpMenuItems),"activate",
	    GTK_SIGNAL_FUNC(disconnect_outputs_node_menu_activate),floor);
    gtk_menu_append(GTK_MENU(floor->popUpMenuOnNode),popUpMenuItems);
    gtk_widget_show(popUpMenuItems);

    floor->activeNode = NULL;

    /*take care of pixmaps*/
    floor->pixmap = NULL;

    /*take care of the drawing area*/
    floor->drawingArea = gtk_drawing_area_new();
    gtk_drawing_area_size(GTK_DRAWING_AREA(floor->drawingArea),850,550);

    /* initial values for some variables */
    floor->connectingToInput = FALSE;
    floor->connectingToOutput = FALSE;
    floor->inputConnection = NULL;
    floor->outputConnection = NULL;
    floor->dragging = FALSE;
    floor->draggingObject = NULL;
    floor->floorFileName[0] = '\0'; /* a string of zero length */
    /* Add the default nodes */
    defaultNodes = node_main_in();
    floor->nodeList = g_list_append(floor->nodeList,defaultNodes);

    defaultNodes = node_main_out();
    floor->nodeList = g_list_append(floor->nodeList,defaultNodes);

    /* signals used to handle backing pixmap */
    gtk_signal_connect(GTK_OBJECT(floor->drawingArea),"expose_event",
	    GTK_SIGNAL_FUNC(floor_expose_event),NULL);
    gtk_signal_connect(GTK_OBJECT(floor->drawingArea),"configure_event",
	    GTK_SIGNAL_FUNC(floor_configure_event),NULL);

    gtk_signal_connect(GTK_OBJECT(floor->drawingArea),"button_release_event",
	    GTK_SIGNAL_FUNC(floor_button_release_event),NULL);
    gtk_signal_connect(GTK_OBJECT(floor->drawingArea),"button_press_event",
	    GTK_SIGNAL_FUNC(floor_button_press_event),NULL);
    gtk_signal_connect(GTK_OBJECT(floor->drawingArea),"motion_notify_event",
	    GTK_SIGNAL_FUNC(floor_motion_notify_event),NULL);

    gtk_widget_set_events (floor->drawingArea, GDK_EXPOSURE_MASK
	    | GDK_LEAVE_NOTIFY_MASK
	    | GDK_BUTTON_PRESS_MASK
	    | GDK_BUTTON_RELEASE_MASK
	    | GDK_POINTER_MOTION_MASK
	    | GDK_POINTER_MOTION_HINT_MASK);

    /*take care of the colors*/
    floor->bgColor = NULL;

    return floor;
}

void connect_with_indexes(Floor *floor,int sourceNode,int sourcePort, int destNode, int destPort){
    NodeConnection *inputNodeConnection,*outputNodeConnection;
    Node *source,*dest;
    source = g_list_nth_data(floor->nodeList,sourceNode);
    dest = g_list_nth_data(floor->nodeList,destNode);
    outputNodeConnection = (NodeConnection *)malloc(sizeof(NodeConnection));
    outputNodeConnection->port = destPort;
    outputNodeConnection->node = dest;
    source->outputConnections[sourcePort] = outputNodeConnection;

    inputNodeConnection = (NodeConnection *)malloc(sizeof(NodeConnection));
    inputNodeConnection->port = sourcePort;
    inputNodeConnection->node = source;
    dest->inputConnections[destPort] = inputNodeConnection;
}

gint floor_save(Floor *floor){
    FILE *fp;
    Node *node;
    GList *nodeNode;
    int i;
    char *tmp_buffer;
    NodeConnection *nodeConnection;
    fp = fopen(floor->floorFileName,"w");
    if(!fp){
	create_message_window("Error trying to open file for writing");
	return FALSE;
    }
    /*print the header*/
    fprintf(fp,"stompboxes2file\n");
    nodeNode = g_list_first(floor->nodeList);
    while(nodeNode){
	node = (Node *)nodeNode->data;
	/* node properties header */
	fprintf(fp,"#\n");
	/* name */
	fprintf(fp,"%s\n",node->so_name);
	/* position */
	fprintf(fp,"%d %d\n",node->position.x,node->position.y);
	/* properties */
	if(!IS_NODE_TYPE_MAIN_INPUT(node->nodeType) && !IS_NODE_TYPE_MAIN_OUTPUT(node->nodeType)){
	    tmp_buffer = node->pluginInfo->get_state(node->pluginData);
	    fprintf(fp,"%s\n",tmp_buffer);
	    free(tmp_buffer);
	}
	else{
	    fprintf(fp,"nan\n");
	}
	/* separator */
	nodeNode = nodeNode->next;
    }
    /* now the connections */
    /* start with the connections marker */
    nodeNode = g_list_first(floor->nodeList);
    while(nodeNode){
	node = (Node *)nodeNode->data;
	for(i=0;i<node->numOutputs;i++){
	    if(node->outputConnections[i]!=NULL){
		nodeConnection = (NodeConnection *)node->outputConnections[i];
		fprintf(fp,"*\n%d %d %d %d\n",g_list_index(floor->nodeList,node),i,g_list_index(floor->nodeList,nodeConnection->node),
			nodeConnection->port);
	    }
	}
	nodeNode = nodeNode->next;
    }
    fclose(fp);
    return TRUE;
}
gint floor_save_with_name(Floor *floor,gchar *name){
    strcpy(floor->floorFileName,name);
    floor_save(floor);
    return TRUE;
}
gint floor_open_with_name(Floor *floor,gchar *name){
    Floor *newFloor;
    int x,y;
    char buffer[256];
    FILE *fd;
    Node *node;
    int connection[4];
    fd = fopen(name,"r");
    if(!fd){
	create_message_window("Could not open file for reading");
	return FALSE;
    }
    newFloor = gui_floor_new();
    fgets(buffer,256,fd);
    buffer[strlen(buffer)-1]='\0';
    if(strcmp(buffer,"stompboxes2file")!=0){
	create_message_window("Invalid Stompboxes2 file\n");
	fclose(fd);
	return FALSE;
    }
    while(fgets(buffer,256,fd)){
	buffer[strlen(buffer)-1] = '\0';
	//fprintf(stderr,"%s\n",buffer);
	if(buffer[0]=='#'){
	    fgets(buffer,256,fd);
	    buffer[strlen(buffer)-1] = '\0';
	    /*the name of the so is stored in buffer*/
	    if(strcmp(buffer,"INPUT")==0){
		fgets(buffer,256,fd);
		sscanf(buffer,"%d %d",&x, &y);
		node = get_main_input_node(newFloor);
		node->position.x = x;
		node->position.y = y;
		fgets(buffer,256,fd);
	    }
	    else if(strcmp(buffer,"OUTPUT")==0){
		fgets(buffer,256,fd);
		sscanf(buffer,"%d %d",&x, &y);
		node = get_main_output_node(newFloor);
		node->position.x = x;
		node->position.y = y;
		fgets(buffer,256,fd);
	    }
	    else{
		node = node_new_with_name(buffer);
		fgets(buffer,256,fd);
		sscanf(buffer,"%d %d",&x, &y);
		node->position.x = x;
		node->position.y = y;
		fgets(buffer,256,fd);
		/* this string has the plugin parameters... have
		 * to do something later */
		node->pluginInfo->set_state(node->pluginData,buffer);
		newFloor->nodeList = g_list_append(newFloor->nodeList,node);
	    }
	}
	else if(buffer[0]=='*'){
	    fgets(buffer,256,fd);
	    sscanf(buffer,"%d %d %d %d",&connection[0],&connection[1],&connection[2],&connection[3]);
	    /* now perform the connection */
	    connect_with_indexes(newFloor,connection[0],connection[1],connection[2],connection[3]);
	}
    }
    fclose(fd);
    /* what i am doing here is a horrible hack
     * what a shame...
     */
    newFloor->drawingArea = currentFloor->drawingArea;
    newFloor->pixmap = currentFloor->pixmap;
    newFloor->bgColor = currentFloor->bgColor;
    newFloor->linksColor = currentFloor->linksColor;
    newFloor->nodesColor = currentFloor->nodesColor;
    newFloor->fontColor = currentFloor->fontColor;
    newFloor->blackColor = currentFloor->blackColor;
    newFloor->whiteColor = currentFloor->whiteColor;
    strcpy(newFloor->floorFileName,name);
    set_current_floor(newFloor);
    /* signals used to handle backing pixmap */
    floor_repaint(newFloor);
    return TRUE;
}
