#include <config.h>
#include <node.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glib.h>
#include <stdio.h>

extern int blocksize;
static int tmpBlocksize;
extern Floor *currentFloor;
extern gboolean isThreadRunning;

void message_window_destroy(GtkObject *object,
			    gpointer user_data) {
    /* release the focus held by the object (message_window) */
    gtk_grab_remove(GTK_WIDGET(object));
}

void close_message_window(GtkButton *button,
			  gpointer user_data) {
    gtk_widget_destroy(GTK_WIDGET(user_data));
}

void create_message_window(char *message) {
    GtkWidget *messageWindow;
    GtkWidget *messageLabel;
    GtkWidget *messageButton;

    /* create the window (a dialog window),
       set the title, and set the window to be modal */
    messageWindow = gtk_dialog_new();
    if (messageWindow == NULL) {
	/* error creating the dialog window */
	//error_handler("Unable to create the base messageWindow");
    }
    gtk_window_set_title(GTK_WINDOW(messageWindow), "Message");
    gtk_window_set_position(GTK_WINDOW(messageWindow), GTK_WIN_POS_CENTER);
    gtk_window_set_modal(GTK_WINDOW(messageWindow), TRUE);

    /* connect the "destroy" signal to a function that will release
       the focus held by the messageWindow */
    gtk_signal_connect_object(GTK_OBJECT(messageWindow), "destroy",
			      GTK_SIGNAL_FUNC(message_window_destroy),
			      GTK_OBJECT(messageWindow));

    /* create the messageLabel, and add it to the packing box */
    messageLabel = gtk_label_new(message);
    if (messageLabel == NULL) {
	/* error creating the message label */
	//error_handler("Unable to create the message window label");
    }
    gtk_misc_set_padding(GTK_MISC(messageLabel), 10, 10);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(messageWindow)->vbox),
		       messageLabel, FALSE, FALSE, 0);
    gtk_widget_show(messageLabel);

    /* create the "Ok" button, connect it with a function to close
       the window when the button is pressed, and add it to the
       dialog box */
    messageButton = gtk_button_new_with_label("OK");
    if (messageButton == NULL) {
	/* error creating the messageWindow button */
	//error_handler("Unable to create the message window button");
    }
    gtk_signal_connect(GTK_OBJECT(messageButton), "clicked",
		       GTK_SIGNAL_FUNC(close_message_window),
		       GTK_OBJECT(messageWindow));
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(messageWindow)->action_area),
		       messageButton, TRUE, TRUE, 0);

    /* set the messageButton to be the default button */
    GTK_WIDGET_SET_FLAGS(messageButton, GTK_CAN_DEFAULT);
    gtk_widget_grab_default(messageButton);

    /* make the button visible */
    gtk_widget_show(messageButton);
    gtk_widget_show(messageWindow);
}

void close_quickstart( GtkWidget *widget,
                        gpointer   data )
{
    gtk_widget_destroy(GTK_WIDGET(data));
}


void show_quickstart_help(){
    GtkWidget *window;
    GtkWidget *box1;
    GtkWidget *box2;
    GtkWidget *hbox;
    GtkWidget *button;
    GtkWidget *separator;
    GtkWidget *table;
    GtkWidget *vscrollbar;
    GtkWidget *text;
    GdkColor color;
    GdkFont *fixed_font;
    GdkColormap *cmap;
    char fileName[256];
    FILE *infile;

    window =  gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_usize (window, 600, 500);
    gtk_window_set_policy (GTK_WINDOW(window), TRUE, TRUE, FALSE);  
    gtk_signal_connect (GTK_OBJECT (window), "destroy",
	    GTK_SIGNAL_FUNC(close_quickstart),
	    window);
    
    gtk_container_set_border_width (GTK_CONTAINER (window), 0);
  
  
    box1 = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), box1);
    gtk_widget_show (box1);
  
  
    box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
    gtk_box_pack_start (GTK_BOX (box1), box2, TRUE, TRUE, 0);
    gtk_widget_show (box2);
  
  
    table = gtk_table_new (2, 2, FALSE);
    gtk_table_set_row_spacing (GTK_TABLE (table), 0, 2);
    gtk_table_set_col_spacing (GTK_TABLE (table), 0, 2);
    gtk_box_pack_start (GTK_BOX (box2), table, TRUE, TRUE, 0);
    gtk_widget_show (table);
  
    /* Create the GtkText widget */
    text = gtk_text_new (NULL, NULL);
    gtk_text_set_editable (GTK_TEXT (text), TRUE);
    gtk_table_attach (GTK_TABLE (table), text, 0, 1, 0, 1,
	    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
	    GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
    gtk_widget_show (text);
    /* Add a vertical scrollbar to the GtkText widget */
    vscrollbar = gtk_vscrollbar_new (GTK_TEXT (text)->vadj);
    gtk_table_attach (GTK_TABLE (table), vscrollbar, 1, 2, 0, 1,
	    GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
    gtk_widget_show (vscrollbar);

    /* Get the system color map and allocate the color red */
    cmap = gdk_colormap_get_system();

    /* Load a fixed font */
    fixed_font = gdk_font_load ("-adobe-courier-medium-r-normal-*-*-120-*-*-m-*-iso8859-1");

    /* Realizing a widget creates a window for it,
     * ready for us to insert some text */
    gtk_widget_realize (text);

    /* Freeze the text widget, ready for multiple updates */
    gtk_text_freeze (GTK_TEXT (text));

    /* Load the file text.c into the text window */

    strcpy(fileName,DOCS_PATH);
    strcat(fileName,"/QUICKSTART");
    infile = fopen(fileName, "r");
    if(!fileName){
	create_message_window("Could not find the Help file\nBad installation ??");
	return;
    }
    if (infile) {
	char buffer[1024];
	int nchars;

	while (1)
	{
	    nchars = fread(buffer, 1, 1024, infile);
	    gtk_text_insert (GTK_TEXT (text), fixed_font, NULL,
		    NULL, buffer, nchars);

	    if (nchars < 1024)
		break;
	}

	fclose (infile);
    }

    /* Thaw the text widget, allowing the updates to become visible */  
    gtk_text_thaw (GTK_TEXT (text));

    separator = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);
    gtk_widget_show (separator);

    box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
    gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, TRUE, 0);
    gtk_widget_show (box2);

    button = gtk_button_new_with_label ("close");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC(close_quickstart),
	    window);
    gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (button);
    gtk_widget_show (button);

    gtk_widget_show (window);
}

gint settings_window_destroy(GtkObject *object,gpointer data){
    gtk_grab_remove(GTK_WIDGET(object));
    gtk_widget_destroy(GTK_WIDGET(data));
}

void set_tmp_blocksize(GtkWidget *widget,gpointer data){
    int value;
    value = GPOINTER_TO_INT(data);
    tmpBlocksize = value;
}

void settings_window_ok(GtkObject *object,gpointer data){
    /* activate the changes */
    if(tmpBlocksize!=blocksize){
	if(isThreadRunning){
	    stop_loop();
	    blocksize = tmpBlocksize;
	    init_loop_on_floor(currentFloor);
	}else
	    blocksize = tmpBlocksize;
	    
    }	

    /* destroy the window */
    gtk_widget_destroy(GTK_WIDGET(data));
}

void show_settings_dialog(){
    GtkWidget *window;
    GtkWidget *frame;
    GtkWidget *vbox;
    GtkWidget *blocksizeBox,*blocksizeFrame;
    GtkWidget *radio1,*radio2,*radio3,*radio4;
    GtkWidget *label;
    GtkWidget *hbox;
    GtkWidget *notebook;
    GtkWidget *okButton;
    GtkWidget *cancelButton;

    tmpBlocksize = blocksize;
    //fprintf(stderr,"%d\n",blocksize);
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(window),"Settings");
    gtk_window_set_position(GTK_WINDOW(window),
                            GTK_WIN_POS_CENTER);
    gtk_window_set_modal(GTK_WINDOW(window), TRUE);
    gtk_container_border_width(GTK_CONTAINER(window), 10);
    gtk_signal_connect_object(GTK_OBJECT(window), "destroy",
	    GTK_SIGNAL_FUNC(settings_window_destroy),
	    GTK_OBJECT(window));
    vbox = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(window),vbox);

    notebook = gtk_notebook_new();
    gtk_box_pack_start(GTK_BOX(vbox),notebook,FALSE,FALSE,0);

    blocksizeFrame = gtk_frame_new("Block Size");
    label = gtk_label_new("Block Size");
    blocksizeBox = gtk_vbox_new(FALSE,0);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),blocksizeBox,label);
    /* Add the radio buttons here */
    radio1 = gtk_radio_button_new_with_label(NULL,"256");
    if(tmpBlocksize==256)
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio1),TRUE);
    gtk_box_pack_start(GTK_BOX(blocksizeBox),radio1,FALSE,FALSE,0);
    radio2 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio1),"1024");
    if(tmpBlocksize==1024)
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio2),TRUE);
    gtk_box_pack_start(GTK_BOX(blocksizeBox),radio2,FALSE,FALSE,0);
    radio3 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio1),"2048");
    if(tmpBlocksize==2048)
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio3),TRUE);
    gtk_box_pack_start(GTK_BOX(blocksizeBox),radio3,FALSE,FALSE,0);
    /*radio4 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio1),"2048");
    if(tmpBlocksize==2048)
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio4),TRUE);
    gtk_box_pack_start(GTK_BOX(blocksizeBox),radio4,FALSE,FALSE,0);
    */
    gtk_signal_connect(GTK_OBJECT(radio1),"pressed",GTK_SIGNAL_FUNC(set_tmp_blocksize),GINT_TO_POINTER(256));
    gtk_signal_connect(GTK_OBJECT(radio2),"pressed",GTK_SIGNAL_FUNC(set_tmp_blocksize),GINT_TO_POINTER(1024));
    gtk_signal_connect(GTK_OBJECT(radio3),"pressed",GTK_SIGNAL_FUNC(set_tmp_blocksize),GINT_TO_POINTER(2048));
    /*gtk_signal_connect(GTK_OBJECT(radio4),"pressed",GTK_SIGNAL_FUNC(set_tmp_blocksize),GINT_TO_POINTER(2048));*/


    /*The buttons at the bottom*/
    hbox = gtk_hbox_new(TRUE,10);
    gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,5);
    okButton = gtk_button_new_with_label("OK");
    gtk_signal_connect(GTK_OBJECT(okButton),"clicked",GTK_SIGNAL_FUNC(settings_window_ok),GTK_OBJECT(window));
    gtk_box_pack_start(GTK_BOX(hbox),okButton,FALSE,TRUE,0);
    cancelButton = gtk_button_new_with_label("Cancel");
    gtk_box_pack_start(GTK_BOX(hbox),cancelButton,FALSE,TRUE,0);
    gtk_signal_connect(GTK_OBJECT(cancelButton),"clicked",GTK_SIGNAL_FUNC(settings_window_destroy),GTK_OBJECT(window));

    gtk_widget_show_all(window);
}
