#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>

#include <gui.h>
#include <plugin.h>
#include <node.h>
#include <oss.h>
#define MAX_BLOCK_SIZE 2048

static int new_pos = 30;

int node_width = 110;
int node_height = 50;
int connection_width = 15;

extern int blocksize;
extern int sampling_frequency;

void update_new_pos(){
    if(new_pos>400) new_pos=30;
    else new_pos+=30;
}
/* This function was created for testing... It should not be used
 * anymore. See node_new_with_name
 */
Node *node_new(){
    /* CONSTRUCTOR */
    Node *node;
    int i;
    node = (Node *)malloc(sizeof(Node));
    /* assign a new position that doesn't overlap. This method is
     * quick'n dirty.
     */
    node->position.x = new_pos;
    node->position.y = new_pos;
    update_new_pos();

    /* XXXX For now, we assume that all nodes have one input and one
     * output
     */
    node->numInputs = 4;
    node->numOutputs = 4;
    node->inputConnections = (void **)calloc(sizeof(void *),node->numInputs);
    node->outputConnections = (void **)calloc(sizeof(void *),node->numOutputs);
    /* initialize the connections */
    for(i=0;i<node->numInputs;i++)
	node->inputConnections[i] = NULL;

    for(i=0;i<node->numOutputs;i++)
	node->outputConnections[i] = NULL;
    /* set the name of the node */
    strcpy(node->name,"ABCDEFGHIJ");
    node->textFont = gdk_font_load("-adobe-courier-medium-r-normal-*-*-120-*-*-m-*-iso8859-1");

    /*define the type of node*/
    node->nodeType = NODE_TYPE_NORMAL | NODE_TYPE_FILTER;
    return node;
}

Node *node_new_with_name(gchar *pluginName){
    Node *node;
    void *handle;
    Plugin *pluginInfo;
    Plugin *(*get_plugin_info)(void);
    gchar fullfilename[256]=PLUGINS_PATH;
    int newLines; /* the number of buffer lines that we MUST allocate memory for */
    int maxLines;
    int i;

    /* initialize the node */
    node = (Node *)malloc(sizeof(Node));
    /* assign a new position that doesn't overlap. This method is
     * quick'n dirty.
     */
    node->position.x = new_pos;
    node->position.y = new_pos;
    update_new_pos();

    /* handle the shared library plugin */
    strcpy(node->so_name,pluginName);
    strcat(fullfilename,"/");
    strcat(fullfilename,pluginName);
    if((handle=dlopen(fullfilename, RTLD_NOW)) == NULL){
	return NULL;
    }
    if((get_plugin_info=dlsym(handle, "get_plugin_info")) != NULL){
	pluginInfo=get_plugin_info();
	strcpy(node->name,pluginInfo->name);
	node->numInputs = pluginInfo->num_inputs;
	node->numOutputs = pluginInfo->num_outputs;
    }
    node->pluginInfo = pluginInfo;
    node->pluginData = node->pluginInfo->init();
    /* initialize the connections */
    node->inputConnections = (void **)calloc(sizeof(void *),node->numInputs);
    node->outputConnections = (void **)calloc(sizeof(void *),node->numOutputs);

    node->textFont = gdk_font_load("-adobe-courier-medium-r-normal-*-*-120-*-*-m-*-iso8859-1");

    /*define the type of node*/
    node->nodeType = NODE_TYPE_NORMAL | NODE_TYPE_FILTER;

    /* set up the bufferlines that this node will use
     * we have to allocate memory for |input - output|
     * number of buffer lines
     */
    maxLines = node->numInputs;
    if(node->numOutputs > maxLines) 
	maxLines = node->numOutputs;
    if(node->numOutputs > node->numInputs){
	newLines = node->numOutputs - node->numInputs;
    }else
	newLines = 0;
    /*fprintf(stderr,"Maxlines = %d, Newlines %d\n",maxLines,newLines);*/
    node->buffers = (short **)calloc(sizeof(short *),maxLines);
    for(i=maxLines-newLines;i<maxLines;i++){
	node->buffers[i] = (short *)calloc(sizeof(short),MAX_BLOCK_SIZE);
	/*fprintf(stderr,"%d\n",i);*/
    }
    
    /* set up the readyBuffers array */
    node->readyBuffers = (gboolean *)calloc(sizeof(gboolean),node->numInputs);
    return node;
}

Node *node_main_in(){
    Node *node;
    int i;
    node = (Node *)malloc(sizeof(Node));
    node->position.x = new_pos;
    node->position.y = new_pos;
    update_new_pos();
    node->numOutputs = 1;
    node->numInputs = 0;
    /* initialize the connections */
    node->inputConnections = (void **)calloc(sizeof(void *),node->numInputs);
    node->outputConnections = (void **)calloc(sizeof(void *),node->numOutputs);
    for(i=0;i<node->numInputs;i++)
	node->inputConnections[i] = NULL;
    for(i=0;i<node->numOutputs;i++)
	node->outputConnections[i] = NULL;

    /*set the name for this node*/
    strcpy(node->name,"INPUT");
    strcpy(node->so_name,"INPUT");

    node->textFont = gdk_font_load("-adobe-courier-medium-r-normal-*-*-120-*-*-m-*-iso8859-1");
    node->buffers = (short **)calloc(sizeof(short *),1);
    node->readyBuffers = (gboolean *)calloc(sizeof(gboolean),1);
    node->nodeType = NODE_TYPE_MAIN_INPUT | NODE_TYPE_PRODUCER;
    return node;
}

Node *node_main_out(){
    Node *node;
    int i;
    node = (Node *)malloc(sizeof(Node));
    node->position.x = new_pos;
    node->position.y = new_pos;
    update_new_pos();
    node->numOutputs = 0;
    node->numInputs = 1;
    /* initialize the connections */
    node->inputConnections = (void **)calloc(sizeof(void *),node->numInputs);
    node->outputConnections = (void **)calloc(sizeof(void *),node->numOutputs);
    for(i=0;i<node->numInputs;i++)
	node->inputConnections[i] = NULL;
    for(i=0;i<node->numOutputs;i++)
	node->outputConnections[i] = NULL;

    /*set the name for this node*/
    strcpy(node->name,"   OUTPUT");
    strcpy(node->so_name,"OUTPUT");
    node->textFont = gdk_font_load("-adobe-courier-medium-r-normal-*-*-120-*-*-m-*-iso8859-1");

    node->nodeType = NODE_TYPE_MAIN_OUTPUT | NODE_TYPE_CONSUMER;
    node->buffers = (short **)calloc(sizeof(short *),1);
    node->readyBuffers = (gboolean *)calloc(sizeof(gboolean),1);
    return node;
}

void node_destroy(Node *node){
    /* DESTRUCTOR */
    int i;
    int maxLines, newLines;
    /* clean the plugin data */
    node->pluginInfo->cleanup(node->pluginData);
    /* clean the line buffers created */
    maxLines = node->numInputs;
    if(node->numOutputs > maxLines) 
	maxLines = node->numOutputs;
    if(node->numOutputs > node->numInputs){
	newLines = node->numOutputs - node->numInputs;
    }else
	newLines = 0;
    for(i=maxLines-newLines;i<maxLines;i++){
	free(node->buffers[i]);
	/*fprintf(stderr,"%d\n",i);*/
    }
    free(node->buffers);
    /* free the readyBuffers array */
    free(node->readyBuffers);
    /* TODO:  free the outputConnections and inputConnections */
    /* free the whole thing */
    free(node);
}

void node_disconnect_outputs(Node *node){
    int i;
    NodeConnection *thisConnection;
    for(i=0;i<node->numOutputs;i++){
	if(node->outputConnections[i]!=NULL){
	    /* disconnect to this node in the other end*/
	    thisConnection = (NodeConnection *)node->outputConnections[i];
	    free((NodeConnection *)thisConnection->node->inputConnections[thisConnection->port]);
	    (NodeConnection *)thisConnection->node->inputConnections[thisConnection->port]=NULL;
	    /* disconnect the connection from here*/
	    free(node->outputConnections[i]);
	    node->outputConnections[i] = NULL;
	}

    }
}

void node_disconnect_inputs(Node *node){
    int i;
    NodeConnection *thisConnection;
    for(i=0;i<node->numInputs;i++){
	if(node->inputConnections[i]!=NULL){
	    thisConnection = (NodeConnection *)node->inputConnections[i];
	    free((NodeConnection *)thisConnection->node->outputConnections[thisConnection->port]);
	    (NodeConnection *)thisConnection->node->outputConnections[thisConnection->port] = NULL;
	    free(node->inputConnections[i]);
	    node->inputConnections[i] = NULL;
	}
    }
}

void draw_node_in_floor(Node *node, Floor *floor){
    int i; 
    Node *otherNode;
    int otherPort;
    if(IS_NODE_TYPE_NORMAL(node->nodeType)){
	/*rectangle for the body*/
	gdk_draw_rectangle(floor->pixmap,floor->nodesColor,
		TRUE,
		node->position.x+connection_width,
		node->position.y,
		node_width-connection_width-connection_width,
		node_height);
	/* the two connection sections */
	gdk_draw_rectangle(floor->pixmap,floor->whiteColor,
		TRUE,
		node->position.x,
		node->position.y,
		connection_width,
		node_height);

	gdk_draw_rectangle(floor->pixmap,floor->whiteColor,
		TRUE,
		node_width + node->position.x - connection_width,
		node->position.y,
		connection_width,
		node_height);

    }
    else if(IS_NODE_TYPE_MAIN_INPUT(node->nodeType)){
	/*rectangle for the body*/
	gdk_draw_rectangle(floor->pixmap,floor->nodesColor,
		TRUE,
		node->position.x,
		node->position.y,
		node_width-connection_width,
		node_height);
	gdk_draw_rectangle(floor->pixmap,floor->whiteColor,
		TRUE,
		node_width + node->position.x - connection_width,
		node->position.y,
		connection_width,
		node_height);

    }
    else if(IS_NODE_TYPE_MAIN_OUTPUT(node->nodeType)){
	gdk_draw_rectangle(floor->pixmap,floor->nodesColor,
		TRUE,
		node->position.x+connection_width,
		node->position.y,
		node_width-connection_width,
		node_height);
	gdk_draw_rectangle(floor->pixmap,floor->whiteColor,
		TRUE,
		node->position.x,
		node->position.y,
		connection_width,
		node_height);

    }
    /* a black frame around everything */
    gdk_draw_rectangle(floor->pixmap,floor->blackColor,
	    FALSE,
	    node->position.x,
	    node->position.y,
	    node_width,node_height);
    /* the name of the node */
    gdk_draw_text(floor->pixmap,node->textFont,
	    floor->blackColor,
	    node->position.x + connection_width + 4,
	    node->position.y + (int) (.5 * node_height) + 4,
	    node->name,strlen(node->name));
	    //node->name,11);

	    /* now draw the outgoing connections and the connection lines*/
	    for(i=0;i<node->numOutputs;i++){
		/* output plug lines */
		gdk_draw_line(floor->pixmap, floor->blackColor,
			node->position.x + node_width - connection_width,
			node->position.y + (int)(((float)node_height / (float)(node->numOutputs+1)) * (i+1)),
			node->position.x + node_width,
			node->position.y + (int)(((float)node_height / (float)(node->numOutputs+1)) * (i+1)));
		if(node->outputConnections[i]!=NULL){
		    /* it is connected somewhere */
		    /* draw the line */
		    otherNode = ((NodeConnection *)node->outputConnections[i])->node;
		    otherPort = ((NodeConnection *)node->outputConnections[i])->port;
		    gdk_draw_line(floor->pixmap, floor->blackColor,
			    node->position.x+node_width,
			    node->position.y + (int)(((float)node_height / (float)(node->numOutputs+1)) * (i+1)),
			    otherNode->position.x,
			    otherNode->position.y + (int)(((float)node_height / (float)(otherNode->numInputs+1)) * (otherPort+1)));
		}
	    }
	    /* input plug lines */
	    for(i=0;i<node->numInputs;i++){
		gdk_draw_line(floor->pixmap, floor->blackColor,
			node->position.x ,
			node->position.y + (int)(((float)node_height / (float)(node->numInputs+1)) * (i+1)),
			node->position.x + connection_width,
			node->position.y + (int)(((float)node_height / (float)(node->numInputs+1)) * (i+1)));

	    }
}

gint point_touches_node(GdkPoint point,Node *node){
    if(point.x > node->position.x && point.x < node->position.x+node_width 
	    && point.y > node->position.y && point.y < node->position.y+node_height)
	return TRUE;
    return FALSE;
}



gint point_touches_node_section(GdkPoint point,Node *node){
    if(point.x > node->position.x && point.x < node->position.x+connection_width
	    && point.y > node->position.y && point.y < node->position.y+node_height)
	return NODE_SECTION_INPUT;
    if(point.x > node->position.x+connection_width && point.x < node->position.x + node_width - connection_width
	    && point.y > node->position.y && point.y < node->position.y+node_height)
	return NODE_SECTION_BODY;
    if(point.x > node->position.x + node_width - connection_width && point.x < node->position.x + node_width
	    && point.y > node->position.y && point.y < node->position.y+node_height)
	return NODE_SECTION_OUTPUT;
    return NODE_SECTION_NONE;
}


gint node_output_available(Node *node){
    int i=0;
    for(i=0;i<node->numOutputs;i++){
	if(node->outputConnections[i] == NULL)
	    return TRUE;
    }
    return FALSE;
}

gint node_input_available(Node *node){
    int i=0;
    for(i=0;i<node->numInputs;i++){
	if(node->inputConnections[i] == NULL)
	    return TRUE;
    }
    return FALSE;
}

gint node_connect_input_to_output(Node *inputNode, Node *outputNode){
    int i,j;
    NodeConnection *inputNodeConnection,*outputNodeConnection;
    for(i=0;i<inputNode->numInputs;i++){
	if(inputNode->inputConnections[i] == NULL){
	    /* this slot is available */
	    inputNodeConnection = (NodeConnection *)malloc(sizeof(NodeConnection));
	    inputNodeConnection->port = i;
	    inputNodeConnection->node = outputNode;
	    inputNode->inputConnections[i] = inputNodeConnection;
	    break;
	}
    }
    for(j=0;j<outputNode->numOutputs;j++){
	if(outputNode->outputConnections[j] == NULL){
	    /* this slot is available */
	    outputNodeConnection = (NodeConnection *)malloc(sizeof(NodeConnection));
	    outputNodeConnection->port = j;
	    outputNodeConnection->node = inputNode;
	    outputNode->outputConnections[j] = outputNodeConnection;
	    break;
	}
    }
    ((NodeConnection *)inputNode->inputConnections[i])->port = j;
    ((NodeConnection *)outputNode->outputConnections[j])->port = i;
    return TRUE;    

}

void node_configure_plugin(Node *node){
    if(IS_NODE_TYPE_NORMAL(node->nodeType))
	node->pluginInfo->configure(node->pluginData);
}


Node *get_main_input_node(Floor *floor){
    GList *nodeNode;
    Node *thisNode;
    nodeNode = g_list_first(floor->nodeList);
    while(nodeNode){
	thisNode = (Node *)nodeNode->data;
	if(IS_NODE_TYPE_MAIN_INPUT(thisNode->nodeType))
	    return thisNode;
	nodeNode = nodeNode->next;
    }
    return NULL;
}
Node *get_main_output_node(Floor *floor){
    GList *nodeNode;
    Node *thisNode;
    nodeNode = g_list_first(floor->nodeList);
    while(nodeNode){
	thisNode = (Node *)nodeNode->data;
	if(IS_NODE_TYPE_MAIN_OUTPUT(thisNode->nodeType))
	    return thisNode;
	nodeNode = nodeNode->next;
    }
    return NULL;
}

void process_node(Node *node, short int **buffers){
    int i;
    if(IS_NODE_TYPE_MAIN_INPUT(node->nodeType)){
	/* is the main input */
	node->processed = TRUE;
	if(node->outputConnections[0]!=NULL){
	    node->buffers = buffers;
	    node_pass_buffers(node,((NodeConnection *)node->outputConnections[0])->node,node->outputConnections[0],0);
	    process_node(((NodeConnection *)node->outputConnections[0])->node,buffers);
	}
    }
    else if(IS_NODE_TYPE_MAIN_OUTPUT(node->nodeType)){
	/* is the main output... 
	 * we need to write to the soundcard here.
	 */
	node->processed = TRUE;
	sound_write_oss(node->buffers[0],blocksize);
	return;
    }
    /* ELSE
     * - Is the node ready for processing?? (i.e. are all the inputs
     * connected ??)
     * - If yes, process and keep spreading through all the outputs.
     * - If not, just return... do nothing.. not our fault.
     */
    else{/* Is an ordinary plugin. Process the data */
	if(node->isReadyForProcessing){
	    node->pluginInfo->process(node->pluginData,node->buffers,blocksize,sampling_frequency);
//	    fprintf(stderr,"processed %s\n",node->name);
	    node->processed = TRUE;
	    /* start spreading through all the outputs*/
	    for(i=0;i<node->numOutputs;i++){
		if(node->outputConnections[i]!=NULL){
	//	    ((NodeConnection *)node->outputConnections[i])->node->buffers[((NodeConnection *)node->outputConnections[i])->port] = node->buffers[i];
		    node_pass_buffers(node,((NodeConnection *)node->outputConnections[i])->node,node->outputConnections[i],i);
		    process_node(((NodeConnection *)node->outputConnections[i])->node,buffers);
		}
	    }
	}
    }

}

void node_pass_buffers(Node *inNode, Node *outNode, NodeConnection *outputConnection, int outPort){
    int i;
    outNode->buffers[outputConnection->port] = inNode->buffers[outPort]; 
    outNode->readyBuffers[outputConnection->port] = TRUE;
    if(1==1){
//	fprintf(stderr,"From %s,%d to %s,%d\n",inNode->name,outPort,outNode->name,outputConnection->port);
    }
    for(i=0;i<outNode->numInputs;i++){
	if(outNode->readyBuffers[i]==FALSE) return;
    }
    outNode->isReadyForProcessing = TRUE;
}

