/* most stuff for this plugin taken from the program gtkgep*/
#include <gtk/gtk.h>
#include <plugin.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#define MAX_DELAY 3000.0
#define MAX_SAMP_FREQ 45000

struct plugin_data
{
    GtkWidget *config_window;
    short int *delay_data;
    int position;
    int total_data_size;
    int sampling_frequency;
    float delay;
    float wet, dry;
    float feedback;
};


static void cleanup(void *data)
{
    struct plugin_data *temp;

    temp=data;
    if(temp->config_window != NULL)
	gtk_widget_destroy(temp->config_window);
    if(temp->delay_data != NULL) free(temp->delay_data);
    free(temp);
}

static void *init(void)
{
    struct plugin_data *temp;

    temp=(struct plugin_data *)malloc(sizeof(struct plugin_data));
    temp->delay=MAX_DELAY / 2.0;
    temp->dry=50.0;
    temp->wet=50.0;
    temp->feedback=50.0;
    temp->position=0;
    temp->sampling_frequency=MAX_SAMP_FREQ;
    temp->total_data_size=(int)(MAX_DELAY / 1000.0 * (float)MAX_SAMP_FREQ);
    temp->delay_data=(short int *)calloc(temp->total_data_size, 2);
    temp->config_window=NULL;
    return temp;
}



static void delay_changed(GtkWidget *adj, gpointer data)
{
    struct plugin_data *temp;
    /*int new_size;*/

    temp=data;

    temp->delay=GTK_ADJUSTMENT(adj)->value;
}

static void wet_changed(GtkWidget *adj, gpointer data)
{
    struct plugin_data *temp;

    temp=data;

    temp->wet=GTK_ADJUSTMENT(adj)->value;
}

static void dry_changed(GtkWidget *adj, gpointer data)
{
    struct plugin_data *temp;

    temp=data;

    temp->dry=GTK_ADJUSTMENT(adj)->value;
}


static void feedback_changed(GtkWidget *adj, gpointer data)
{
    struct plugin_data *temp;

    temp=data;

    temp->feedback=GTK_ADJUSTMENT(adj)->value;
}


static void config_window_destroy(GtkWidget *w, gpointer data)
{
    struct plugin_data *temp;

    temp=data;

    if(temp->config_window != NULL)
	gtk_widget_destroy(temp->config_window);

    temp->config_window=NULL;
}


static void configure(void *data)
{
    struct plugin_data *temp;
    GtkWidget *vbox;
    GtkWidget *hbox;
    GtkWidget *label;	
    GtkObject *delay_adj, *dry_adj, *feedback_adj;
    GtkWidget *delay_bar, *dry_bar, *feedback_bar;
    GtkWidget *wet_bar;
    GtkObject *wet_adj;

    temp=data;

    if(temp->config_window != NULL) return;

    temp->config_window=gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_window_set_title(GTK_WINDOW(temp->config_window), "Digital Delay");
    gtk_widget_set_usize(temp->config_window, 300, 150);
    gtk_signal_connect(GTK_OBJECT(temp->config_window), "destroy", GTK_SIGNAL_FUNC(config_window_destroy), data);

    delay_adj=gtk_adjustment_new(temp->delay, 5.0, MAX_DELAY, 1.0, 1.0, 0.0);
    gtk_signal_connect(GTK_OBJECT(delay_adj), "value_changed", GTK_SIGNAL_FUNC(delay_changed), data);
    dry_adj=gtk_adjustment_new(temp->dry, 0.0, 100.0, 1.0, 1.0, 0.0);
    gtk_signal_connect(GTK_OBJECT(dry_adj), "value_changed", GTK_SIGNAL_FUNC(dry_changed), data);
    wet_adj=gtk_adjustment_new(temp->wet, 0.0, 100.0, 1.0, 1.0, 0.0);
    gtk_signal_connect(GTK_OBJECT(wet_adj), "value_changed", GTK_SIGNAL_FUNC(wet_changed), data);	
    feedback_adj=gtk_adjustment_new(temp->feedback, 0.0, 100.0, 1.0, 1.0, 0.0);
    gtk_signal_connect(GTK_OBJECT(feedback_adj), "value_changed", GTK_SIGNAL_FUNC(feedback_changed), data);


    hbox=gtk_hbox_new(FALSE, 0);

    vbox=gtk_vbox_new(FALSE, 0);
    gtk_widget_show(vbox);
    label=gtk_label_new("Delay (ms)");
    gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 10);
    gtk_widget_show(label);
    label=gtk_label_new("Dry volume (%)");
    gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 10);
    gtk_widget_show(label);
    label=gtk_label_new("Wet volume (%)");
    gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 10);
    gtk_widget_show(label);	
    label=gtk_label_new("Feedback (%)");
    gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 10);
    gtk_widget_show(label);

    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
    gtk_widget_show(hbox);

    vbox=gtk_vbox_new(FALSE, 0);
    gtk_widget_show(vbox);
    delay_bar=gtk_hscale_new(GTK_ADJUSTMENT(delay_adj));
    gtk_scale_set_digits(GTK_SCALE(delay_bar), 0);
    gtk_scale_set_draw_value(GTK_SCALE(delay_bar), TRUE);
    gtk_box_pack_start(GTK_BOX(vbox), delay_bar, TRUE, TRUE, 0);

    dry_bar=gtk_hscale_new(GTK_ADJUSTMENT(dry_adj));
    gtk_scale_set_digits(GTK_SCALE(dry_bar), 0);
    gtk_scale_set_draw_value(GTK_SCALE(dry_bar), TRUE);
    gtk_box_pack_start(GTK_BOX(vbox), dry_bar, TRUE, TRUE, 0);

    wet_bar=gtk_hscale_new(GTK_ADJUSTMENT(wet_adj));
    gtk_scale_set_digits(GTK_SCALE(wet_bar), 0);
    gtk_scale_set_draw_value(GTK_SCALE(wet_bar), TRUE);
    gtk_box_pack_start(GTK_BOX(vbox), wet_bar, TRUE, TRUE, 0);	


    feedback_bar=gtk_hscale_new(GTK_ADJUSTMENT(feedback_adj));
    gtk_scale_set_digits(GTK_SCALE(feedback_bar), 0);
    gtk_scale_set_draw_value(GTK_SCALE(feedback_bar), TRUE);
    gtk_box_pack_start(GTK_BOX(vbox), feedback_bar, TRUE, TRUE, 0);

    gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE ,0);

    gtk_widget_show(delay_bar);
    gtk_widget_show(dry_bar);
    gtk_widget_show(wet_bar);	
    gtk_widget_show(feedback_bar);
    gtk_widget_show(vbox);

    gtk_container_add(GTK_CONTAINER(temp->config_window), hbox);
    gtk_widget_show(temp->config_window);
}


static int process(void *data, short int **smpdata, int size, int sampling_frequency)
{
    struct plugin_data *pdata;
    int sample_in;
    register int sample_out;
    int feedback_temp;
    int delayed_sample_nr;
    int feedback_int, wet_int, dry_int;
    register int i;
    short int *channel = smpdata[0];

    pdata=data;

    if(sampling_frequency != pdata->sampling_frequency)
    {
	pdata->sampling_frequency=sampling_frequency;
	memset(pdata->delay_data, 0, pdata->total_data_size * 2);
    }

    delayed_sample_nr=pdata->position - (int)(pdata->delay / 1000.0 * (float)sampling_frequency);
    if(delayed_sample_nr < 0) delayed_sample_nr+=pdata->total_data_size;
    feedback_int=(int)(pdata->feedback * 32768.0 / 100.0);
    wet_int=(int)(pdata->wet * 32768.0 / 100.0);
    dry_int=(int)(pdata->dry * 32768.0 / 100.0);	

    for(i=0; i < (size >> 1); i++)
    {
	sample_in=channel[i];
	sample_out=pdata->delay_data[delayed_sample_nr++];
	if(delayed_sample_nr >= pdata->total_data_size) delayed_sample_nr=0;		
	feedback_temp=sample_in + ((sample_out * feedback_int) >> 15);

	if(feedback_temp > 32767) feedback_temp=32767;
	else if(feedback_temp < -32768) feedback_temp=-32768;

	pdata->delay_data[pdata->position++]=(short int)feedback_temp;
	if(pdata->position >= pdata->total_data_size) pdata->position=0;
	sample_out=((sample_in * dry_int) >> 15) + ((sample_out * wet_int) >> 15);

	if(sample_out > 32767) sample_out=32767;
	else if(sample_out < -32768) sample_out=-32768;

	channel[i]=sample_out;
    }

    return size;
}


static char *get_state(void *data){
    char *state_string;
    struct plugin_data *plug = (struct plugin_data *)data;
    state_string = (char *)calloc(sizeof(char),256);
    sprintf(state_string,"%f %f %f %f",plug->delay,plug->wet,plug->dry,plug->feedback);
    return state_string;
}


static int set_state(void *data, char *state_string){
    struct plugin_data *plug = (struct plugin_data *)data;
    sscanf(state_string,"%f %f %f %f",&plug->delay,&plug->wet,&plug->dry,&plug->feedback);
    return 1;
}

Plugin plugin_info={"Delay",
    cleanup,
    init,
    configure,
    process,
    get_state,
    set_state,
    1,
    1};


Plugin *get_plugin_info(void)
{
    return &plugin_info;
}
